#import <Foundation/Foundation.h>
#ifdef __NeXT__
  #import <EOControl/EOKeyValueCoding.h>
#endif

#define TEST(expr, expect) NSLog(@"%s ==> %@  (expect: %s)", #expr, expr, #expect)


void KVCTestMain(void)
{
    NSArray *array = [NSArray arrayWithContentsOfFile: @"KVC.plist"];
    NSDictionary *dict = [NSDictionary dictionaryWithObject: array forKey: @"allObjects"];

    NSCAssert(array != nil, @"Failed to load KVC.plist");

    TEST([dict valueForKeyPath: @"allObjects.@sum.locations.@sum.population"], 1240800);   /* Total population */
    TEST([dict valueForKeyPath: @"allObjects.@avg.locations.@sum.population"], 413600);    /* Average state population */
    TEST([dict valueForKeyPath: @"allObjects.@avg.locations.@avg.population"], 178094.4...);    /* Average of states' average location populations */
    
    TEST([array valueForKeyPath: @"@sum.locations.@sum.population"], 1240800);   /* Total population */
    TEST([array valueForKeyPath: @"@avg.locations.@sum.population"], 413600);    /* Average state population */
    TEST([array valueForKeyPath: @"@avg.locations.@avg.population"], 178094.4...);    /* Average of states' average location populations */

    /* Behaves differently on Cocoa, as the key is interpreted as [array valueForKey: @"sum.locations.@sum.population"] (using first example) */
    TEST([array valueForKey: @"@sum.locations.@sum.population"], 1240800);   /* Total population */
    TEST([array valueForKey: @"@avg.locations.@sum.population"], 413600);    /* Average state population */
    TEST([array valueForKey: @"@avg.locations.@avg.population"], 178094.4...);    /* Average of states' average location populations */   
}


int main (int argc, const char * argv[])
{
    NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

    KVCTestMain();

    [pool release];
    return 0;
}

/*
 Output on Cocoa (Mac OS X 10.5.2 PowerPC):

 2008-03-13 23:08:04.188 KVC[18431:10b] [dict valueForKeyPath: @"allObjects.@sum.locations.@sum.population"] ==> 1240800  (expect: 1240800)
 2008-03-13 23:08:04.190 KVC[18431:10b] [dict valueForKeyPath: @"allObjects.@avg.locations.@sum.population"] ==> 413600  (expect: 413600)
 2008-03-13 23:08:04.192 KVC[18431:10b] [dict valueForKeyPath: @"allObjects.@avg.locations.@avg.population"] ==> 178094.444444444444444444444444444443333  (expect: 178094.4...)
 2008-03-13 23:08:04.192 KVC[18431:10b] [array valueForKeyPath: @"@sum.locations.@sum.population"] ==> 1240800  (expect: 1240800)
 2008-03-13 23:08:04.193 KVC[18431:10b] [array valueForKeyPath: @"@avg.locations.@sum.population"] ==> 413600  (expect: 413600)
 2008-03-13 23:08:04.193 KVC[18431:10b] [array valueForKeyPath: @"@avg.locations.@avg.population"] ==> 178094.444444444444444444444444444443333  (expect: 178094.4...)
 2008-03-13 23:08:04.194 KVC[18431:10b] *** Terminating app due to uncaught exception 'NSUnknownKeyException', reason: '[<NSCFArray 0x105a80> valueForUndefinedKey:]: this class is not key value coding-compliant for the key sum.locations.@sum.population.'
 
 */

/*
 Output on OPENSTEP 4.2 (m68k) with EOF 2.1:

 -- Task 'KVC' started on 2008-03-13 22:08:57 -0600 --
 Mar 13 22:09:00 KVC[335] [dict valueForKeyPath: allObjects.@sum.locations.@sum.population ] ==> 1240800  (expect: 1240800)
 Mar 13 22:09:01 KVC[335] [dict valueForKeyPath: allObjects.@avg.locations.@sum.population ] ==> 413600  (expect: 413600)
 Mar 13 22:09:01 KVC[335] [dict valueForKeyPath: allObjects.@avg.locations.@avg.population ] ==> 178094.444444444444444444444444444443333  (expect: 178094.4...)
 Mar 13 22:09:01 KVC[335] [array valueForKeyPath: @sum.locations.@sum.population ] ==> 1240800  (expect: 1240800)
 Mar 13 22:09:01 KVC[335] [array valueForKeyPath: @avg.locations.@sum.population ] ==> 413600  (expect: 413600)
 Mar 13 22:09:01 KVC[335] [array valueForKeyPath: @avg.locations.@avg.population ] ==> 178094.444444444444444444444444444443333  (expect: 178094.4...)
 Mar 13 22:09:02 KVC[335] [array valueForKey: @sum.locations.@sum.population ] ==> 1240800  (expect: 1240800)
 Mar 13 22:09:02 KVC[335] [array valueForKey: @avg.locations.@sum.population ] ==> 413600  (expect: 413600)
 Mar 13 22:09:02 KVC[335] [array valueForKey: @avg.locations.@avg.population ] ==> 178094.444444444444444444444444444443333  (expect: 178094.4...)

 -- Task 'KVC' finished on 2008-03-13 22:09:02 -0600 --
 
 */
